@@ -2,7 +2,7 @@
 """
 A test harness for gnupg.py.
 
-Copyright (C) 2008-2022 Vinay Sajip. All rights reserved.
+Copyright (C) 2008-2023 Vinay Sajip. All rights reserved.
 """
 import argparse
 import json
@@ -16,9 +16,14 @@ import sys
 import tempfile
 import unittest
 
+try:
+    unicode
+except NameError:
+    unicode = str
+
 try:
     from unittest import skipIf
-except ImportError:
+except ImportError:  # pragma: no cover
     # For now, for Python < 2.7
     def skipIf(condition, message):
         if not condition:
@@ -30,7 +35,7 @@ except ImportError:
 import gnupg
 
 __author__ = 'Vinay Sajip'
-__date__ = '$23-Aug-2022 16:37:02$'
+__date__ = '$12-Dec-2023 07:57:43$'
 
 ALL_TESTS = True
 
@@ -208,18 +213,18 @@ debug ipc
 
 ENABLE_TOFU = 'ENABLE_TOFU' in os.environ
 
-if ENABLE_TOFU:
+if ENABLE_TOFU:  # pragma: no cover
     GPG_CONFIG = 'trust-model tofu+pgp\ntofu-default-policy unknown\n'
 
 
 def prepare_homedir(hd):
-    if not os.path.isdir(hd):
+    if not os.path.isdir(hd):  # pragma: no cover
         os.makedirs(hd)
     os.chmod(hd, 0x1C0)
     fn = os.path.join(hd, 'gpg-agent.conf')
     with open(fn, 'w') as f:
         f.write(AGENT_CONFIG)
-    if ENABLE_TOFU:
+    if ENABLE_TOFU:  # pragma: no cover
         fn = os.path.join(hd, 'gpg.conf')
         with open(fn, 'w') as f:
             f.write(GPG_CONFIG)
@@ -232,7 +237,7 @@ class GPGTestCase(unittest.TestCase):
         logger.debug('-- %s starting ---------------------------' % ident)
         if 'STATIC_TEST_HOMEDIR' not in os.environ:
             hd = tempfile.mkdtemp(prefix='keys-')
-        else:
+        else:  # pragma: no cover
             hd = os.path.join(os.getcwd(), 'keys')
             if os.path.exists(hd):
                 self.assertTrue(os.path.isdir(hd), 'Not a directory: %s' % hd)
@@ -261,8 +266,7 @@ class GPGTestCase(unittest.TestCase):
     def test_environment(self):
         "Test the environment by ensuring that setup worked"
         hd = self.homedir
-        self.assertTrue(
-            os.path.exists(hd) and os.path.isdir(hd), 'Not an existing directory: %s' % hd)
+        self.assertTrue(os.path.exists(hd) and os.path.isdir(hd), 'Not an existing directory: %s' % hd)
 
     def test_list_keys_initial(self):
         "Test that initially there are no keys"
@@ -341,9 +345,8 @@ class GPGTestCase(unittest.TestCase):
         uids = key['uids']
         self.assertEqual(len(uids), 1)
         uid = uids[0]
-        self.assertEqual(
-            uid, 'urn:uuid:731c22c4-830f-422f-80dc-14a9fdae8c19 '
-            '(dummy comment) <test.name@example.com>')
+        self.assertEqual(uid, 'urn:uuid:731c22c4-830f-422f-80dc-14a9fdae8c19 '
+                         '(dummy comment) <test.name@example.com>')
 
     def test_key_generation_with_escapes(self):
         "Test that key generation handles escape characters"
@@ -364,15 +367,14 @@ class GPGTestCase(unittest.TestCase):
         uids = key['uids']
         self.assertEqual(len(uids), 1)
         uid = uids[0]
-        self.assertEqual(
-            uid, 'Test Name (Funny chars: '
-            '\r\n\x0c\x0b\x00\x08) <test.name@example.com>')
+        self.assertEqual(uid, 'Test Name (Funny chars: '
+                         '\r\n\x0c\x0b\x00\x08) <test.name@example.com>')
 
     @unittest.skipIf(os.name == 'nt', 'Test requires POSIX-style permissions')
     def test_key_generation_failure(self):
-        if self.gpg.version < (2, 0):
+        if self.gpg.version < (2, 0):  # pragma: no cover
             raise unittest.SkipTest('gpg 1.x hangs in this test')
-        if not os.path.exists('rokeys'):
+        if not os.path.exists('rokeys'):  # pragma: no cover
             os.mkdir('rokeys')
         os.chmod('rokeys', 0o400)  # no one can write/search this directory
         gpg = gnupg.GPG(gnupghome='rokeys', gpgbinary=GPGBINARY)
@@ -410,20 +412,16 @@ class GPGTestCase(unittest.TestCase):
             'name_comment': 'NIST P-384',
         }
         cmd = self.gpg.gen_key_input(**params)
-        for s in ('Key-Type: ECDSA', 'Key-Curve: nistp384', 'Subkey-Type: ECDH',
-                  'Subkey-Curve: nistp384', 'Name-Comment: NIST P-384'):
+        for s in ('Key-Type: ECDSA', 'Key-Curve: nistp384', 'Subkey-Type: ECDH', 'Subkey-Curve: nistp384',
+                  'Name-Comment: NIST P-384'):
             self.assertTrue('%s\n' % s in cmd)
         self.assertFalse('Key-Length: ' in cmd)
 
     def test_add_subkey(self):
         "Test that subkeys can be added"
-        if self.gpg.version[0] < 2:
+        if self.gpg.version[0] < 2:  # pragma: no cover
             raise unittest.SkipTest('Feature unavailable in GnuPG 1.x')
-        master_key = self.generate_key('Charlie',
-                                       'Clark',
-                                       'gamma.com',
-                                       passphrase='123',
-                                       with_subkey=False)
+        master_key = self.generate_key('Charlie', 'Clark', 'gamma.com', passphrase='123', with_subkey=False)
         self.assertEqual(0, master_key.returncode, 'Non-zero return code')
 
         result = self.gpg.add_subkey(master_key=master_key.fingerprint,
@@ -435,13 +433,9 @@ class GPGTestCase(unittest.TestCase):
 
     def test_add_subkey_with_invalid_key_type(self):
         "Test that subkey generation handles invalid key type"
-        if self.gpg.version[0] < 2:
+        if self.gpg.version[0] < 2:  # pragma: no cover
             raise unittest.SkipTest('Feature unavailable in GnuPG 1.x')
-        master_key = self.generate_key('Charlie',
-                                       'Clark',
-                                       'gamma.com',
-                                       passphrase='123',
-                                       with_subkey=False)
+        master_key = self.generate_key('Charlie', 'Clark', 'gamma.com', passphrase='123', with_subkey=False)
         self.assertEqual(0, master_key.returncode, 'Non-zero return code')
 
         result = self.gpg.add_subkey(master_key=master_key.fingerprint,
@@ -456,13 +450,9 @@ class GPGTestCase(unittest.TestCase):
 
     def test_deletion_subkey(self):
         "Test that subkey deletion works"
-        if self.gpg.version[0] < 2:
+        if self.gpg.version[0] < 2:  # pragma: no cover
             raise unittest.SkipTest('Feature unavailable in GnuPG 1.x')
-        master_key = self.generate_key('Charlie',
-                                       'Clark',
-                                       'gamma.com',
-                                       passphrase='123',
-                                       with_subkey=False)
+        master_key = self.generate_key('Charlie', 'Clark', 'gamma.com', passphrase='123', with_subkey=False)
         self.assertEqual(0, master_key.returncode, 'Non-zero return code')
 
         subkey = self.gpg.add_subkey(master_key=master_key.fingerprint,
@@ -484,10 +474,7 @@ class GPGTestCase(unittest.TestCase):
 
         self.assertTrue(is_list_with_len(private_keys, 1), '1-element list expected')
         self.assertEqual(len(secret_key_info['subkeys']), 1, '1-element list expected')
-        result = self.gpg.delete_keys(subkey.fingerprint,
-                                      secret=True,
-                                      passphrase='123',
-                                      exclamation_mode=True)
+        result = self.gpg.delete_keys(subkey.fingerprint, secret=True, passphrase='123', exclamation_mode=True)
         result = self.gpg.delete_keys(subkey.fingerprint, exclamation_mode=True)
         self.assertEqual(0, result.returncode, 'Non-zero return code')
 
@@ -506,15 +493,11 @@ class GPGTestCase(unittest.TestCase):
 
     def test_list_subkey_after_generation(self):
         "Test that after subkey generation, the generated subkey is available"
-        if self.gpg.version[0] < 2:
+        if self.gpg.version[0] < 2:  # pragma: no cover
             raise unittest.SkipTest('Feature unavailable in GnuPG 1.x')
         self.test_list_keys_initial()
 
-        master_key = self.generate_key('Charlie',
-                                       'Clark',
-                                       'gamma.com',
-                                       passphrase='123',
-                                       with_subkey=False)
+        master_key = self.generate_key('Charlie', 'Clark', 'gamma.com', passphrase='123', with_subkey=False)
         self.assertEqual(0, master_key.returncode, 'Non-zero return code')
 
         subkey_sign = self.gpg.add_subkey(master_key=master_key.fingerprint,
@@ -624,7 +607,7 @@ class GPGTestCase(unittest.TestCase):
         self.assertEqual(info['type'], 'ssb')
 
         # Now do the same test, but using keyring and secret_keyring arguments
-        if self.gpg.version < (2, 1):
+        if self.gpg.version < (2, 1):  # pragma: no cover
             pkn = 'pubring.gpg'
             skn = 'secring.gpg'
         else:
@@ -635,7 +618,7 @@ class GPGTestCase(unittest.TestCase):
         hd = self.homedir
         if os.name == 'posix':
             pkn = os.path.join(hd, pkn)
-            if skn:
+            if skn:  # pragma: no cover
                 skn = os.path.join(hd, skn)
         gpg = gnupg.GPG(gnupghome=hd, gpgbinary=GPGBINARY, keyring=pkn, secret_keyring=skn)
         logger.debug('Using keyring and secret_keyring arguments')
@@ -681,8 +664,7 @@ class GPGTestCase(unittest.TestCase):
         public_keys = gpg.list_keys(keys=['Donna', 'Barbara'])
         self.assertEqual(0, public_keys.returncode, 'Non-zero return code')
         actual = get_names(public_keys.key_map)
-        expected = set(
-            ['Barbara Brown <barbara.brown@beta.com>', 'Donna Davis <donna.davis@delta.com>'])
+        expected = set(['Barbara Brown <barbara.brown@beta.com>', 'Donna Davis <donna.davis@delta.com>'])
         self.assertEqual(actual, expected)
 
     def test_key_trust(self):
@@ -702,6 +684,7 @@ class GPGTestCase(unittest.TestCase):
             ('TRUST_FULLY', 'f'),
             ('TRUST_ULTIMATE', 'u'),
             ('TRUST_UNDEFINED', 'q'),
+            ('TRUST_EXPIRED', 'e'),
         )
         for param, expected in cases:
             gpg.trust_keys(fingerprints, param)
@@ -723,25 +706,34 @@ class GPGTestCase(unittest.TestCase):
         self.assertTrue(is_list_with_len(keys, 1), 'importing test signed key')
         sigs = self.gpg.list_keys(keys=['18897CA2'], sigs=True)[0]['sigs']
         logger.debug('testing self-signature')
-        self.assertTrue(('BC6F350FB3D04076', 'Joshua Calvert (A test user) <jc@example.com>',
-                         '13x') in sigs)
+        self.assertTrue(('BC6F350FB3D04076', 'Joshua Calvert (A test user) <jc@example.com>', '13x') in sigs)
         logger.debug('testing subkey self-signature')
-        self.assertTrue(('BC6F350FB3D04076', 'Joshua Calvert (A test user) <jc@example.com>',
-                         '18x') in sigs)
+        self.assertTrue(('BC6F350FB3D04076', 'Joshua Calvert (A test user) <jc@example.com>', '18x') in sigs)
         logger.debug('testing other signature')
-        self.assertTrue(('057CCF658C074FDA', 'Winston Smith (A test user) <winston@example.com>',
-                         '10x') in sigs)
+        self.assertTrue(('057CCF658C074FDA', 'Winston Smith (A test user) <winston@example.com>', '10x') in sigs)
 
     def test_scan_keys(self):
         "Test that external key files can be scanned"
         # Don't use SkipTest for now, as not available for Python < 2.7
-        if self.gpg.version < (2, 1):
+        if self.gpg.version < (2, 1):  # pragma: no cover
             expected = set([
                 'Andrew Able (A test user) <andrew.able@alpha.com>',
                 'Barbara Brown (A test user) <barbara.brown@beta.com>',
                 'Charlie Clark (A test user) <charlie.clark@gamma.com>',
             ])
-            for fn in ('test_pubring.gpg', 'test_secring.gpg'):
+            test_files = ('test_pubring.gpg', 'test_secring.gpg')
+            key_fn = None
+        else:
+            expected = set([
+                'Gary Gross (A test user) <gary.gross@gamma.com>',
+                'Danny Davis (A test user) <danny.davis@delta.com>',
+            ])
+            fd, key_fn = tempfile.mkstemp(prefix='pygpg-test-')
+            os.write(fd, KEYS_TO_IMPORT.encode('ascii'))
+            os.close(fd)
+            test_files = (key_fn, )
+        try:
+            for fn in test_files:
                 logger.debug('scanning keys in %s', fn)
                 data = self.gpg.scan_keys(fn)
                 self.assertEqual(0, data.returncode, 'Non-zero return code')
@@ -749,6 +741,24 @@ class GPGTestCase(unittest.TestCase):
                 for d in data:
                     uids.add(d['uids'][0])
                 self.assertEqual(uids, expected)
+        finally:
+            if key_fn:
+                os.remove(key_fn)
+
+    def test_scan_keys_mem(self):
+        "Test that external keys in memory can be scanned"
+        expected = set([
+            'Gary Gross (A test user) <gary.gross@gamma.com>',
+            'Danny Davis (A test user) <danny.davis@delta.com>',
+        ])
+        for key in (KEYS_TO_IMPORT, ):
+            logger.debug('testing scan_keys')
+            data = self.gpg.scan_keys_mem(key)
+            self.assertEqual(0, data.returncode, 'Non-zero return code')
+            uids = set()
+            for d in data:
+                uids.add(d['uids'][0])
+            self.assertEqual(uids, expected)
 
     def test_encryption_and_decryption(self):
         "Test that encryption and decryption works"
@@ -761,7 +771,7 @@ class GPGTestCase(unittest.TestCase):
         gpg = self.gpg
         if gnupg._py3k:
             data = 'Hello, André!'
-        else:
+        else:  # pragma: no cover
             data = unicode('Hello, André', gpg.encoding)
         data = data.encode(gpg.encoding)
         result = gpg.encrypt(data, barbara)
@@ -788,24 +798,9 @@ class GPGTestCase(unittest.TestCase):
         self.assertEqual(data, ddata.data, 'Round-trip must work')
         # Test symmetric encryption
         data = 'chippy was here'
-        self.assertRaises(ValueError,
-                          gpg.encrypt,
-                          data,
-                          None,
-                          passphrase='bbr\x00own',
-                          symmetric=True)
-        self.assertRaises(ValueError,
-                          gpg.encrypt,
-                          data,
-                          None,
-                          passphrase='bbr\rown',
-                          symmetric=True)
-        self.assertRaises(ValueError,
-                          gpg.encrypt,
-                          data,
-                          None,
-                          passphrase='bbr\nown',
-                          symmetric=True)
+        self.assertRaises(ValueError, gpg.encrypt, data, None, passphrase='bbr\x00own', symmetric=True)
+        self.assertRaises(ValueError, gpg.encrypt, data, None, passphrase='bbr\rown', symmetric=True)
+        self.assertRaises(ValueError, gpg.encrypt, data, None, passphrase='bbr\nown', symmetric=True)
         result = gpg.encrypt(data, None, passphrase='bbrown', symmetric=True)
         self.assertEqual(0, result.returncode, 'Non-zero return code')
         edata = str(result)
@@ -870,6 +865,7 @@ class GPGTestCase(unittest.TestCase):
         gpg = self.gpg
         result = gpg.import_keys(KEYS_TO_IMPORT)
         self.assertEqual(0, result.returncode, 'Non-zero return code')
+        self.assertTrue(bool(result))
         self.assertEqual(result.summary(), '2 imported')
         public_keys = gpg.list_keys()
         self.assertEqual(0, public_keys.returncode, 'Non-zero return code')
@@ -887,7 +883,7 @@ class GPGTestCase(unittest.TestCase):
         self.assertEqual(0, match, 'Keys must match')
         # Generate a key so we can test exporting private keys
         key = self.do_key_generation()
-        if self.gpg.version < (2, 1):
+        if self.gpg.version < (2, 1):  # pragma: no cover
             passphrase = None
         else:
             passphrase = 'bbrown'
@@ -936,32 +932,21 @@ class GPGTestCase(unittest.TestCase):
         key = self.generate_key('Andrew', 'Able', 'alpha.com')
         if gnupg._py3k:
             data = 'Hello, André!'
-        else:
+        else:  # pragma: no cover
             data = unicode('Hello, André', self.gpg.encoding)
         data = data.encode(self.gpg.encoding)
-        self.assertRaises(ValueError,
-                          self.gpg.sign,
-                          data,
-                          keyid=key.fingerprint,
-                          passphrase='bbr\x00own')
-        self.assertRaises(ValueError,
-                          self.gpg.sign,
-                          data,
-                          keyid=key.fingerprint,
-                          passphrase='bbr\rown')
-        self.assertRaises(ValueError,
-                          self.gpg.sign,
-                          data,
-                          keyid=key.fingerprint,
-                          passphrase='bbr\nown')
+        self.assertRaises(ValueError, self.gpg.sign, data, keyid=key.fingerprint, passphrase='bbr\x00own')
+        self.assertRaises(ValueError, self.gpg.sign, data, keyid=key.fingerprint, passphrase='bbr\rown')
+        self.assertRaises(ValueError, self.gpg.sign, data, keyid=key.fingerprint, passphrase='bbr\nown')
         sig = self.gpg.sign(data, keyid=key.fingerprint, passphrase='bbrown')
         self.assertFalse(sig, 'Bad passphrase should fail')
         sig = self.gpg.sign(data, keyid=key.fingerprint, passphrase='aable')
         self.assertEqual(0, sig.returncode, 'Non-zero return code')
         self.assertTrue(sig, 'Good passphrase should succeed')
-        if sig.username:  # not set in recent versions of GnuPG e.g. 2.2.5
+        if sig.username:  # pragma: no cover
+            # not set in recent versions of GnuPG e.g. 2.2.5
             self.assertTrue(sig.username.startswith('Andrew Able'))
-        if sig.key_id:
+        if sig.key_id:  # pragma: no cover
             self.assertTrue(key.fingerprint.endswith(sig.key_id))
         self.assertTrue(sig.hash_algo)
         logger.debug('verification start')
@@ -1032,11 +1017,7 @@ class GPGTestCase(unittest.TestCase):
         key = self.generate_key('Andrew', 'Able', 'alpha.com')
         data_file = open(self.test_fn, 'rb')
         sig_file = self.test_fn + '.asc'
-        sig = self.gpg.sign_file(data_file,
-                                 keyid=key.fingerprint,
-                                 passphrase='aable',
-                                 detach=True,
-                                 output=sig_file)
+        sig = self.gpg.sign_file(data_file, keyid=key.fingerprint, passphrase='aable', detach=True, output=sig_file)
         self.assertEqual(0, sig.returncode, 'Non-zero return code')
         data_file.close()
         self.assertTrue(sig, 'File signing should succeed')
@@ -1053,20 +1034,16 @@ class GPGTestCase(unittest.TestCase):
         finally:
             os.remove(sig_file)
         self.assertEqual(0, verified.returncode, 'Non-zero return code')
-        if key.fingerprint != verified.fingerprint:
+        if key.fingerprint != verified.fingerprint:  # pragma: no cover
             logger.debug('key: %r', key.fingerprint)
             logger.debug('ver: %r', verified.fingerprint)
         self.assertEqual(key.fingerprint, verified.fingerprint, 'Fingerprints must match')
 
     def test_subkey_signature_file(self):
         "Test that signing and verification works via the GPG output for subkeys"
-        if self.gpg.version[0] < 2:
+        if self.gpg.version[0] < 2:  # pragma: no cover
             raise unittest.SkipTest('Feature unavailable in GnuPG 1.x')
-        master_key = self.generate_key('Charlie',
-                                       'Clark',
-                                       'gamma.com',
-                                       passphrase='123',
-                                       with_subkey=False)
+        master_key = self.generate_key('Charlie', 'Clark', 'gamma.com', passphrase='123', with_subkey=False)
         self.assertEqual(0, master_key.returncode, 'Non-zero return code')
 
         subkey = self.gpg.add_subkey(master_key=master_key.fingerprint,
@@ -1078,11 +1055,7 @@ class GPGTestCase(unittest.TestCase):
 
         data_file = open(self.test_fn, 'rb')
         sig_file = self.test_fn + '.asc'
-        sig = self.gpg.sign_file(data_file,
-                                 keyid=subkey.fingerprint,
-                                 passphrase='123',
-                                 detach=True,
-                                 output=sig_file)
+        sig = self.gpg.sign_file(data_file, keyid=subkey.fingerprint, passphrase='123', detach=True, output=sig_file)
 
         self.assertEqual(0, sig.returncode, 'Non-zero return code')
         data_file.close()
@@ -1100,7 +1073,7 @@ class GPGTestCase(unittest.TestCase):
         finally:
             os.remove(sig_file)
         self.assertEqual(0, verified.returncode, 'Non-zero return code')
-        if subkey.fingerprint != verified.fingerprint:
+        if subkey.fingerprint != verified.fingerprint:  # pragma: no cover
             logger.debug('key: %r', subkey.fingerprint)
             logger.debug('ver: %r', verified.fingerprint)
         self.assertEqual(subkey.fingerprint, verified.fingerprint, 'Fingerprints must match')
@@ -1269,9 +1242,11 @@ class GPGTestCase(unittest.TestCase):
         finally:
             shutil.rmtree(d, ignore_errors=True)
 
-    # @skipIf(os.name == 'nt', 'Test not suitable for Windows')
-    def test_search_keys(self):
+    # This test does nothing on CI because it often leads to failures due to
+    # external servers being down
+    def test_search_keys(self):  # pragma: no cover
         "Test that searching for keys works"
+
         if 'NO_EXTERNAL_TESTS' not in os.environ:
             r = self.gpg.search_keys('<vinay_sajip@hotmail.com>')
             self.assertEqual(0, r.returncode, 'Non-zero return code')
@@ -1381,7 +1356,7 @@ class GPGTestCase(unittest.TestCase):
         self.assertTrue(pubkey1)
         if gpg.version >= (2, 1):
             passphrase = 'pp1'
-        else:
+        else:  # pragma: no cover
             passphrase = None
         seckey1 = gpg.export_keys(fp1, secret=True, passphrase=passphrase)
         self.assertTrue(seckey1)
@@ -1395,7 +1370,7 @@ class GPGTestCase(unittest.TestCase):
         result = gpg.delete_keys(fp1)
         self.assertEqual(2, result.returncode, 'Unexpected return code')
         self.assertEqual(str(result), 'Must delete secret key first')
-        if gpg.version < (2, 1):
+        if gpg.version < (2, 1):  # pragma: no cover
             # Doesn't work on 2.1, and can't use SkipTest due to having
             # to support older Pythons
             result = gpg.delete_keys(fp1, secret=True, passphrase=passphrase)
@@ -1449,7 +1424,7 @@ class GPGTestCase(unittest.TestCase):
         gpg = self.gpg
         if gnupg._py3k:
             data = 'Hello, André!'
-        else:
+        else:  # pragma: no cover
             data = unicode('Hello, André', gpg.encoding)
         data = data.encode(gpg.encoding)
         encrypted = gpg.encrypt(data, barbara)
@@ -1527,38 +1502,84 @@ class GPGTestCase(unittest.TestCase):
         finally:
             os.remove(fn)
 
+    def test_multiple_signatures(self):
+        gpg = self.gpg
+        key1 = self.generate_key('Andrew', 'Able', 'alpha.com')
+        key2 = self.generate_key('Barbara', 'Brown', 'beta.com')
+        data = b'signed data'
+        sig1 = gpg.sign(data, keyid=key1.fingerprint, passphrase='aable', detach=True)
+        sig2 = gpg.sign(data, keyid=key2.fingerprint, passphrase='bbrown', detach=True)
+        # Combine the signatures, then verify
+        fd, fn = tempfile.mkstemp(prefix='pygpg-test-')
+        os.write(fd, sig1.data)
+        os.write(fd, sig2.data)
+        os.close(fd)
+        try:
+            verified = self.gpg.verify_data(fn, data)
+            sig_info = verified.sig_info
+            self.assertEqual(len(sig_info), 2)
+            actual = set(d['fingerprint'] for d in sig_info.values())
+            expected = set((key1.fingerprint, key2.fingerprint))
+            self.assertEqual(actual, expected)
+        finally:
+            os.remove(fn)
+
+    def test_multiple_signatures_one_invalid(self):
+        gpg = self.gpg
+        key1 = self.generate_key('Andrew', 'Able', 'alpha.com')
+        key2 = self.generate_key('Barbara', 'Brown', 'beta.com')
+        data = b'signed data'
+        other_data = b'other signed data'
+        sig1 = gpg.sign(data, keyid=key1.fingerprint, passphrase='aable', detach=True)
+        sig2 = gpg.sign(other_data, keyid=key2.fingerprint, passphrase='bbrown', detach=True)
+        # Combine the signatures, then verify
+        fd, fn = tempfile.mkstemp(prefix='pygpg-test-')
+        os.write(fd, sig1.data)
+        os.write(fd, sig2.data)
+        os.close(fd)
+        try:
+            verified = self.gpg.verify_data(fn, data)
+            sig_info = verified.sig_info
+            self.assertEqual(len(sig_info), 1)
+            actual = set(d['fingerprint'] for d in sig_info.values())
+            expected = set([key1.fingerprint])
+            self.assertEqual(actual, expected)
+            problems = verified.problems
+            self.assertEqual(len(problems), 1)
+            d = problems[0]
+            self.assertEqual(d['status'], 'signature bad')
+            self.assertTrue(key2.fingerprint.endswith(d['keyid']))
+        finally:
+            os.remove(fn)
+
 
 TEST_GROUPS = {
     'sign':
     set(['test_signature_verification', 'test_signature_file', 'test_subkey_signature_file']),
     'crypt':
     set([
-        'test_encryption_and_decryption', 'test_file_encryption_and_decryption',
-        'test_filenames_with_spaces', 'test_invalid_outputs', 'test_no_such_key'
+        'test_encryption_and_decryption', 'test_file_encryption_and_decryption', 'test_filenames_with_spaces',
+        'test_invalid_outputs', 'test_no_such_key'
     ]),
     'key':
     set([
-        'test_deletion', 'test_import_and_export', 'test_list_keys_after_generation',
-        'test_list_signatures', 'test_key_generation_with_invalid_key_type',
-        'test_key_generation_with_escapes', 'test_key_generation_input',
-        'test_key_generation_with_colons', 'test_search_keys', 'test_scan_keys', 'test_key_trust',
-        'test_add_subkey', 'test_add_subkey_with_invalid_key_type', 'test_deletion_subkey',
+        'test_deletion', 'test_import_and_export', 'test_list_keys_after_generation', 'test_list_signatures',
+        'test_key_generation_with_invalid_key_type', 'test_key_generation_with_escapes', 'test_key_generation_input',
+        'test_key_generation_with_colons', 'test_search_keys', 'test_scan_keys', 'test_scan_keys_mem',
+        'test_key_trust', 'test_add_subkey', 'test_add_subkey_with_invalid_key_type', 'test_deletion_subkey',
         'test_list_subkey_after_generation'
     ]),
     'import':
     set(['test_import_only', 'test_doctest_import_keys']),
     'basic':
-    set([
-        'test_environment', 'test_list_keys_initial', 'test_nogpg', 'test_make_args',
-        'test_quote_with_shell'
-    ]),
+    set(['test_environment', 'test_list_keys_initial', 'test_nogpg', 'test_make_args', 'test_quote_with_shell']),
     'test':
-    set(['test_passing_paths']),
+    set(['test_multiple_signatures_one_invalid']),
 }
 
 
 def suite(args=None):
-    if args is None:
+    if args is None:  # pragma: no cover
         args = sys.argv[1:]
     if not args:
         result = unittest.TestLoader().loadTestsFromTestCase(GPGTestCase)
@@ -1593,7 +1614,7 @@ def main():
     verbosity = 2 if options.verbose else 1
     results = unittest.TextTestRunner(verbosity=verbosity).run(tests)
     failed = not results.wasSuccessful()
-    if failed and 'TOXENV' in os.environ and os.name != 'posix':
+    if failed and 'TOXENV' in os.environ and os.name != 'posix':  # pragma: no cover
         os.system('type test_gnupg.log')
     return failed
 
